=begin

= apache/asp-run.rb

Copyright (C) 2003 Grgoire Lejeune <greg@webtime-project.net>
All rights reserved.

== Overview

Apache::AspRun handles eRuby files by Asp.

== Example of .htaccess

AuthName "Ruby/ASP"

RubySetEnv Global /var/www/tests/ruby-asp/global-asa

RubyRequire apache/aspruby-run
<Files ~ (\.asp)>
  SetHandler ruby-object
  RubyHandler Apache::AspRun.instance
</Files>

=end

require "singleton"
require "tempfile"
require "asp/common"
require "asp/compile"
require "asp/globalasa"
require "asp/response"
require "asp/request"
require "asp/session"
require "asp/application"
require "asp/server"

module Apache
  class AspRun
    include Singleton

    def handler(r)
      if r.method_number == M_OPTIONS
        r.allowed |= (1 << M_GET)
        r.allowed |= (1 << M_POST)
        return DECLINED
      end
      if r.finfo.mode == 0
        return NOT_FOUND
      end

      s = r.server( )
#      c = r.connection( )
#      t = r.headers_in( )
      
      r.setup_cgi_env
      
      btime = Time::now
      
      oGAsa = Asp::GlobalASA::new( ENV["Global"], r ) 
      if oGAsa.getAsaCode == false 
        print __FILE__," L",__LINE__," :: Call ASP.Error\n";
        return OK
      end

      $Request = Asp::Request::new( r )
      $Response = Asp::Response::new( r )
      $Session = Asp::Session::new( r )
      $Server = Asp::Server::new( r )

      filename = r.filename.dup
      filename.untaint
      
      code = compile(filename)
      prerun(r)
      run(code, filename)
      postrun(r, btime)

      return OK
    end

    private

    def initialize
      @compiler = AspCompiler.new
      @compiler.put_cmd = 'print'
    end

    def compile(filename)
      open(filename) do |f|
        return @compiler.compile(f.read)
      end
    end

    def prerun(r)
      if self.private_methods.member?( "script_OnStart" ) == true
        script_OnStart
      end
      Apache.chdir_file(r.filename)
    end

    def run(code, filename)
      tmpdir = (ENV['TMPDIR'] || ENV['TMP'] || ENV['TEMP'] || '/tmp').untaint
      file = Tempfile.new(File.basename(filename) + ".", tmpdir)
      begin
        file.print(code)
        file.close
        load(file.path, true)
      ensure
        file.close(true)
      end
    end

    def postrun(r, btime)
      if self.private_methods.member?( "script_OnEnd" ) == true
        script_OnEnd

        etime = Time::now
        ctime = etime - btime
        print "<!-- Ruby/ASP v",Asp::Common::version," served page in ",ctime," seconds -->"
      end

      r.content_type = format( "%s; charset=%s", $Response.contentType, $Response.charset )
      $Response.getHeader.each_pair{ |key,value|
        r.headers_out[key] = value
      }
      $Response.addCookiesHeader.each { |cookie|
        r.headers_out.add( "Set-Cookie", cookie )
      }
      sessionCookie = $Session.addSessionCookieHeader
      if sessionCookie.nil? == false
        r.headers_out.add( "Set-Cookie", sessionCookie )
      end
      r.send_http_header
    end

  end
end
